#!/usr/bin/perl
# Ralf's Chat - a Perl webchat script
# Copyright 1998-2002 Ralf Gueldemeister <ralf@ralfchat.com>
#
# This file is part of Ralf's Chat.
#
# Ralf's Chat is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Ralf's Chat is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ralf's Chat; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

require "./config.cgi";
require "./shared.cgi";
require "./$lang_dir/$language.cgi";
require "./$html_dir/setup.cgi";

$script_name = "setup.cgi";

# Main Part
##########################################

my @setup_vars = (
    'masterpassword','admin_email_address','mailprogramme','mail_on_new_registration','logtype','max_users','days_to_keep_memos','flocking','international_nicks','language','days_del_old_users',
    'data_dir','html_dir','lang_dir','log_dir','cmd_dir','www_dir',
    'updatefrequency','min_update_freq','clear_old_msgs','message_limit','new_msg_on_top','reset_private_msg_select','show_users_on_login','max_input_length','posting_time','posting_time_format','time_difference','sa_usercolors',
#<DBI>
    'streaming',    
#</DBI>
    'script_name','script2_name','scriptpm_name','scriptsa_name','scriptsu_name','scriptgu_name',
#<DBI>
    'scriptst_name',
#</DBI>
    'data{\'messages\'}','data{\'stillalive\'}','data{\'nicks\'}','data{\'userinfo\'}','data{\'banned\'}','data{\'memo\'}',
#<DBI>
    'use_dbi','dbi_user','dbi_password','dbi_database','dbi_host','dbi_dsn','use_primary_keys',
#</DBI>
    'html_bodytag','css_file','html_title','banner_picture','banner_link','banner_url','logout_url','input_field_size',
);
my %setup_desc = (
    'masterpassword' => 'What would you like your masterpassword to be set to? This is a very important thing for you to set. The masterpassword gets used for many things. Some modules require this password in order to get administrators passwords and for just this standard installation, the masterpassword is used to give yourself administration rights. I suggest that you don\'t give this password to anybody!',
    'admin_email_address' => 'What is your email address? This is used when people get their passwords recovered. This is what will appear in "from" and if you choose to be emailed whenever a new user registers.',
    'mailprogramme' => 'Path to the mail-transport-agent',
    'mail_on_new_registration' => 'set to 1 if you want to be mailed when a new user registers',
    'logtype' => 'The type of log: 0 - no log, 1 - registrations, 2 - +logins, 3 - +messages, 4-6 - +debug',
    'max_users' => 'the maximal number of users in the chat at the same time, set to -1 if unlimited',
    'days_to_keep_memos' => 'Number of days to keep memos',
    'flocking' => 'set to 1 if you want flocking (file lock), this is not supported by Windows NT (set to 0)',
    'international_nicks' => '0 - nick may only contain letters and numbers, 1 - nick may contain any character except ";/ :\?*<>|\'',
    'language' => 'language',
    'days_del_old_users' => 'number of days after old users will be deleted - set to -1 to never delete any users automatically',
  
    'data_dir' => 'data directory',
    'html_dir' => 'html directory',
    'lang_dir' => 'language directory',
    'log_dir' => 'directory for log files - must be writable by the script',
    'cmd_dir' => 'directory for additional commands',
    'www_dir' => 'www directory - has to be inside the www accessible area of your server (not in the "cgi-bin" directory), contains images, style sheets ...',
  
    'updatefrequency' => 'chat refresh delay',
    'min_update_freq' => 'the minimum update frequency',
    'clear_old_msgs' => 'set to 1 if you want that old msgs are deleted when the chat is empty',
    'message_limit' => 'number of maximal messages in the chat window',
    'new_msg_on_top' => 'set to 1 if you want new messages printed on top, to work around the msie bug (default value for new users)',
    'reset_private_msg_select' => 'set to 1 if you want the select field for private msgs to be reset to "to all users" after msg was sent',
    'show_users_on_login' => 'set to 1 if you want to see the currently chatting users on the login page',
    'max_input_length' => 'maximum length of nickname and password',
    'posting_time' => 'set to 1 if you want posting time',
    'posting_time_format' => 'customizable posting time format - see \'man date\' for details of the format',
    'time_difference' => 'time difference between server and local time in hours (may be negative)',
    'sa_usercolors' => 'set to 1 if you want to use the user colors in the stillalive frame (users online)',
    'streaming' => 'default for streaming mode, streaming is only available if you use the DBI',

    'script_name' => 'filename of the script like "chat.cgi"',
    'script2_name' => 'filename of the second script like "chat2.cgi"',
    'scriptpm_name' => 'filename of the post message script', 
    'scriptsa_name' => 'filename of the still alive script', 
    'scriptsu_name' => 'filename of the show users script',
    'scriptgu_name' => 'filename of the goto url script',
#<DBI>
    'scriptst_name' => 'filename of the stream script',
#</DBI>
    'data{\'messages\'}' => '',
    'data{\'stillalive\'}' => '',
    'data{\'nicks\'}' => '',
    'data{\'userinfo\'}' => '',
    'data{\'banned\'}' => '',
    'data{\'memo\'}' => '',
    
#<DBI>
    'use_dbi' => 'set to 1 if you want to use the DBI',
    'dbi_user' => 'user for DBI',
    'dbi_password' => 'password for DBI',
    'dbi_database' => 'database',
    'dbi_host' => 'host',
    'dbi_dsn' => 'DSN (Data Source Name for the DBI)',
    'use_primary_keys' => 'set to one if your DBMS supports primary keys (eg. MySQL)',
#</DBI>

    'html_bodytag' => 'The bodytag for html files',
    'css_file' => 'The style sheet definition used for all html files',
    'html_title' => 'The title of your chat',
    'banner_picture' => 'The banner picture',
    'banner_link' => 'The banner link',
    'banner_url' => 'The banner url, you may set it to your banner rotation script, set to "" if you don\'t want any banner',
    'logout_url' => 'the url for the logout-link',
    'input_field_size' => 'the size of the msg input field',
);

my %setup_cat = (
    'masterpassword' => 'General Settings',
    'data_dir' => 'Directory Names',
    'updatefrequency' => 'Look And Feel',
    'script_name' => 'Script Names',
    'data{\'messages\'}' => 'Files/Tables',
    'use_dbi' => 'DBI (Data Base Interface)',
    'html_bodytag' => 'HTML-Stuff',
);

my %setup_vars_select = ( 
   'logtype'	=> ['0 - no log','1 - registrations','2 - +logins','3 - +messages','4 - +debug','5 - +debug','6 - +debug'],
    'flocking'	=> ['off','on'],
    'international_nicks'	=> ['off','on'],
    'mail_on_new_registration'	=> ['off','on'],
    'clear_old_msgs' => ['off','on'],
    'new_msg_on_top' => ['off','on'],
    'reset_private_msg_select' => ['off','on'],
    'show_users_on_login' => ['off','on'],
    'posting_time' => ['off','on'],
    'sa_usercolors' => ['off','on'],
    'streaming' => ['off','on'],
    'use_dbi' => ['off','on'],
    'use_primary_keys' => ['off','on'],
);

my %setup_vars_select_full = ( # use full element as value, not just its number
    'language'	=> [&setup_get_langs],
    'css_file'	=> [&setup_get_css],
);

&setup_html_header;
&action;
&setup_html_footer;

exit;

sub action {
    if ($query{'action'} and $query{'mpw'} eq $masterpassword) {
	if ($query{'action'} eq 'config_save') {
        &setup_config_save;
	} elsif ($query{'action'} eq 'config_load') {
	    &setup_config_load;
	} elsif ($query{'action'} eq 'perlpath') {
	    &setup_perlpath;
	}
    } else {
    	&setup_html_menu;
    }
}

sub setup_perlpath {
    my @required = (
	'chat.cgi',
	'chat2.cgi',
	'chatgu.cgi',
	'chatpm.cgi',
	'chatsa.cgi',
	'chatsu.cgi',
	'config.cgi',
	'dbi.cgi',
	'shared.cgi',
	'stream.cgi',
	'testusers.cgi',
    );
    print "<B>$html{'setup'}{'cfrf'}</B><BR>\n";
    my $errors = 0;
    foreach my $file (@required) {
	my $error = 0;
	open(FILE, "<$file") or $error = 1;
	close(FILE);

	if ($error) {
	    print "$file $html{'setup'}{'nf'}<BR>\n";
	    $errors++;
	}
    }
    
    if ($errors) {
	return;
    }

    open (THIS,"<$script_name");
    my $shebang = <THIS>;
    close (THIS);
    
    chomp($shebang);
    
    foreach my $file (@required) {
	    open(TEMP,"<$file");
	    @LINES = <TEMP>;
	    close(TEMP);
    
	    if (substr($LINES[0],0,2) eq "#!" && not substr($LINES[0],0,length $shebang) eq $shebang) {
		    $LINES[0] = $shebang."\n";
    
		    open(TEMP,">$file");
		    print TEMP @LINES;
		    close(TEMP);
		    } 
	    }
    
    print "Success!<BR>\nNow for the configuration...<BR>\n";
}

sub setup_config_load {
    open(CONFIG,"<config.cgi");
    my @config = <CONFIG>;
    close CONFIG;

    print << "[END]";
<INPUT TYPE=HIDDEN NAME="action" VALUE="config_save">
<TABLE CELLPADDING=10>
[END]

    my ($setup_var,$setup_var_value);
    foreach $setup_var (@setup_vars) {
	$setup_var_value = '';
	foreach (@config) {
	    if (/^\s*\$$setup_var\s*=\s*("|'|)?(.*)\1\s*;/) {
		$setup_var_value = $2;		
		$setup_var_value =~ s/\\(\\)?/$1/g;
		$setup_var_value =~ s/"/&quot;/g;
		$setup_var_value =~ s/</&lt;/g;
		last;
	    }
	}
	if ($setup_cat{$setup_var}) {
	    print "<TR><TD COLSPAN=3><H3>$setup_cat{$setup_var}</H3></TD></TR>\n";
	}
	print "<TR><TD VALIGN=\"top\"><B>$setup_var</B></TD><TD VALIGN=\"top\">\n";
	if ($setup_vars_select{$setup_var}) {
	    print "<SELECT NAME=\"var_$setup_var\">";
	    my $temp_value = 0;
	    foreach (@{$setup_vars_select{$setup_var}}) {
		if ($temp_value == $setup_var_value) {
		    print "<OPTION VALUE=\"$temp_value\" SELECTED>$_</OPTION>\n";
		} else {
		    print "<OPTION VALUE=\"$temp_value\">$_</OPTION>\n";
		}
		$temp_value++;
	    }
	    print "</SELECT>\n";
	} elsif ($setup_vars_select_full{$setup_var}) {
	    print "<SELECT NAME=\"var_$setup_var\">";
	    foreach (@{$setup_vars_select_full{$setup_var}}) {
		if ($_ eq $setup_var_value) {
		    print "<OPTION VALUE=\"$_\" SELECTED>$_</OPTION>\n";
		} else {
		    print "<OPTION VALUE=\"$_\">$_</OPTION>\n";
		}
	    }
	    print "</SELECT>\n";
	} else {
	    print "<INPUT TYPE=TEXT NAME=\"var_$setup_var\" VALUE=\"$setup_var_value\">";
	}
	print "</TD><TD VALIGN=\"top\">$setup_desc{$setup_var}</TD></TR>";
    }
    print "</TABLE>\n";
}

sub setup_config_save {
    open(CONFIG,"<config.cgi");
    my @config = <CONFIG>;
    close CONFIG;
    
    my ($setup_var, $setup_varset, $setup_var_query, $setup_var_value);
    foreach $setup_var (@setup_vars) {
	$setup_varset = '';
	$setup_var_query = "var_$setup_var";
	$setup_var_value = $query{$setup_var_query};
	$setup_var_value =~ s/("|'|@|\\)/\\$1/g;
	foreach (@config) {
	    if (/^\s*\$$setup_var\s*=\s*("|'|)?(.*)\1\s*;/) {
		$_ = "\$$setup_var = \"$setup_var_value\";\n";
		print "changing $setup_varset $setup_var to $setup_var_value<BR>\n";
		$setup_varset++;
	    } elsif (/^1;\n/ and not $setup_varset) {
		$_ = "\$$setup_var = \"$setup_var_value\";\n\n1;\n";
		print "adding $setup_var = $setup_var_value<BR>\n";
	    }
	}
    }
    
    open(CONFIG,">config.cgi");
    foreach (@config) {
	print CONFIG;
    }
    close CONFIG;
    &setup_html_menu;
}

sub setup_get_langs {
    my @langs;
    opendir(DIR, $lang_dir);
    while (my $lang_file = readdir(DIR)) {
	    if (-f "$lang_dir/$lang_file" and $lang_file =~ s/\.(cgi|pl)$//i) {
	        push(@langs, $lang_file);
	    }
    }
    closedir DIR;
    return @langs;
}

sub setup_get_css {
    my @css;
    opendir(DIR, "$www_dir/css");
    while (my $css_file = readdir(DIR)) {
	    if (-f "$www_dir/css/$css_file" and $css_file =~ /\.css$/i) {
	        push(@css, "\$www_dir/css/$css_file");
	    }
    }
    closedir DIR;
    return @css;
}
